@@ -3065,6 +3065,36 @@ void escape_shell_arg(char *src, size_t len, char *dst) {
 	*ptr++ = 0;
 }
 
+void escape_json(char *src, size_t len, char *dst) {
+
+	size_t i;
+	char *ptr = dst;
+
+	for (i = 0; i < len; i++) {
+		if (src[i] == '\t') {
+			*ptr++ = '\\';
+			*ptr++ = 't';
+		}
+		else if (src[i] == '\n') {
+			*ptr++ = '\\';
+			*ptr++ = 'n';
+		}
+		else if (src[i] == '\r') {
+			*ptr++ = '\\';
+			*ptr++ = 'r';
+		}
+		else if (src[i] == '"') {
+			*ptr++ = '\\';
+			*ptr++ = '"';
+		}
+		else {
+			*ptr++ = src[i];
+		}
+	}
+
+	*ptr++ = 0;
+}
+
 void http_url_decode(char *buf, uint16_t * len, char *dst) {
 
 	uint16_t i;
@@ -3221,14 +3251,22 @@ char *uwsgi_chomp(char *str) {
 	return str;
 }
 
-char *uwsgi_tmpname(char *base, char *id) {
-	char *template = uwsgi_concat3(base, "/", id);
-	if (mkstemp(template) < 0) {
-		free(template);
-		return NULL;
+int uwsgi_tmpfd() {
+	char *tmpdir = getenv("TMPDIR");
+	if (!tmpdir) {
+		tmpdir = "/tmp";
 	}
+	char *template = uwsgi_concat2(tmpdir, "/uwsgiXXXXXX");
+	int fd = mkstemp(template);
+	unlink(template);
+	free(template);
+	return fd;
+}
 
-	return template;
+FILE *uwsgi_tmpfile() {
+	int fd = uwsgi_tmpfd();
+	if (fd < 0) return NULL;
+	return fdopen(fd, "w+");
 }
 
 int uwsgi_file_to_string_list(char *filename, struct uwsgi_string_list **list) {
